@@ -145,6 +145,7 @@ typedef struct {
 	GtkWidget *system_db_button;
 	GtkWidget *remote_db_button;
 	GtkWidget *remote_db_entry;
+	GtkWidget *db_name_entry;
 } UIData;
 
 static void
@@ -152,11 +153,10 @@ destroy_ui_data (gpointer data)
 {
 	UIData *ui = data;
 
-	if (ui) {
+	if (ui && ui->vbox)
 		gtk_widget_destroy (ui->vbox);
-		g_object_unref (ui->source);
-		g_free (ui);
-	}
+
+	g_free (ui);
 }
 
 static void
@@ -166,7 +166,9 @@ on_user_db_toggled (GtkToggleButton *button, gpointer *user_data)
 
 	if (gtk_toggle_button_get_active (button)) {
 		e_source_set_property (ui->source, "couchdb_instance", "user");
+		e_source_set_property (ui->source, "couchdb_database", "contacts");
 		gtk_widget_set_sensitive (ui->remote_db_entry, FALSE);
+		gtk_widget_set_sensitive (ui->db_name_entry, FALSE);
 	}
 }
 
@@ -178,6 +180,7 @@ on_system_db_toggled (GtkToggleButton *button, gpointer *user_data)
 	if (gtk_toggle_button_get_active (button)) {
 		e_source_set_property (ui->source, "couchdb_instance", "system");
 		gtk_widget_set_sensitive (ui->remote_db_entry, FALSE);
+		gtk_widget_set_sensitive (ui->db_name_entry, TRUE);
 	}
 }
 
@@ -189,24 +192,34 @@ on_remote_db_toggled (GtkToggleButton *button, gpointer *user_data)
 	if (gtk_toggle_button_get_active (button)) {
 		e_source_set_property (ui->source, "couchdb_instance", "remote");
 		gtk_widget_set_sensitive (ui->remote_db_entry, TRUE);
+		gtk_widget_set_sensitive (ui->db_name_entry, TRUE);
 	}
 }
 
-static on_remote_db_changed (GtkEditable *entry, gpointer user_data)
+static void
+on_remote_db_changed (GtkEditable *entry, gpointer user_data)
 {
 	UIData *ui = (UIData *) user_data;
 
 	e_source_set_property (ui->source, "couchdb_remote_server", gtk_entry_get_text (GTK_ENTRY (entry)));
 }
 
+static void
+on_db_name_changed (GtkEditable *entry, gpointer user_data)
+{
+	UIData *ui = (UIData *) user_data;
+
+	e_source_set_property (ui->source, "couchdb_database", gtk_entry_get_text (GTK_ENTRY (entry)));
+}
+
 GtkWidget *
 plugin_couchdb_contacts (EPlugin *epl, EConfigHookItemFactoryData *data)
 {
 	ESource *source;
         ESourceGroup *group;
 	const gchar *base_uri;
-	GtkWidget *parent, *parent_vbox;
-	GtkWidget *table, *label;
+	GtkWidget *parent;
+	GtkWidget *table, *label, *parent_vbox;
 	UIData *ui;
 	const gchar *property;
 	EABConfigTargetSource *t = (EABConfigTargetSource *) data->target;
@@ -216,14 +229,14 @@ plugin_couchdb_contacts (EPlugin *epl, EConfigHookItemFactoryData *data)
 
         base_uri = e_source_group_peek_base_uri (group);
 
-        g_object_set_data (G_OBJECT (epl), "wwidget", NULL);
+        g_object_set_data (G_OBJECT (epl), "cwidget", NULL);
 
-	if (strcmp(base_uri, COUCHDB_BASE_URI) != 0)
+	if (strcmp (base_uri, COUCHDB_BASE_URI) != 0)
                 return NULL;
 
 	/* Build up the UI */
 	ui = g_new0 (UIData, 1);
-	ui->source = g_object_ref (source);
+	ui->source = t->source;
 
 	parent = data->parent;
 	parent_vbox = gtk_widget_get_ancestor (gtk_widget_get_parent (parent), GTK_TYPE_VBOX);
@@ -236,31 +249,47 @@ plugin_couchdb_contacts (EPlugin *epl, EConfigHookItemFactoryData *data)
 	gtk_misc_set_alignment (GTK_MISC (label), 0.0, 0.0);
 	gtk_box_pack_start (GTK_BOX (ui->vbox), label, FALSE, FALSE, 0);
 
-	table = gtk_table_new (3, 2, FALSE);
+	table = gtk_table_new (4, 3, FALSE);
 	gtk_box_pack_start (GTK_BOX (ui->vbox), table, TRUE, TRUE, 0);
 
-	ui->user_db_button = gtk_radio_button_new_with_label (NULL, _("User local database"));
-	gtk_table_attach (GTK_TABLE (table), ui->user_db_button, 0, 2, 0, 1,
+	label = gtk_label_new ("   ");
+	gtk_table_attach (GTK_TABLE (table), label, 0, 1, 0, 1, GTK_FILL, GTK_FILL, 0, 0);
+
+	ui->user_db_button = gtk_radio_button_new_with_label (NULL, _("Desktop CouchDB"));
+	gtk_table_attach (GTK_TABLE (table), ui->user_db_button, 1, 3, 0, 1,
 			  GTK_EXPAND | GTK_FILL, GTK_FILL, 3, 3);
 
 	ui->system_db_button = gtk_radio_button_new_with_label (
 		gtk_radio_button_get_group (GTK_RADIO_BUTTON (ui->user_db_button)),
-		_("System wide database"));
-	gtk_table_attach (GTK_TABLE (table), ui->system_db_button, 0, 2, 1, 2,
+		_("System-wide CouchDB"));
+	gtk_table_attach (GTK_TABLE (table), ui->system_db_button, 1, 3, 1, 2,
 			  GTK_EXPAND | GTK_FILL, GTK_FILL, 3, 3);
 
 	ui->remote_db_button = gtk_radio_button_new_with_label (
 		gtk_radio_button_get_group (GTK_RADIO_BUTTON (ui->user_db_button)),
-		_("Remote database"));
-	gtk_table_attach (GTK_TABLE (table), ui->remote_db_button, 0, 1, 2, 3,
+		_("Remote CouchDB server"));
+	gtk_table_attach (GTK_TABLE (table), ui->remote_db_button, 1, 2, 2, 3,
 			  GTK_EXPAND | GTK_FILL, GTK_FILL, 3, 3);
 	ui->remote_db_entry = gtk_entry_new ();
-	gtk_table_attach (GTK_TABLE (table), ui->remote_db_entry, 1, 2, 2, 3,
+	gtk_table_attach (GTK_TABLE (table), ui->remote_db_entry, 2, 3, 2, 3,
+			  GTK_EXPAND | GTK_FILL, GTK_FILL, 3, 3);
+
+	label = gtk_label_new (_("Database name:"));
+	gtk_table_attach (GTK_TABLE (table), label, 1, 2, 3, 4,
+			  GTK_EXPAND | GTK_FILL, GTK_FILL, 3, 3);
+	ui->db_name_entry = gtk_entry_new ();
+	gtk_table_attach (GTK_TABLE (table), ui->db_name_entry, 2, 3, 3, 4,
 			  GTK_EXPAND | GTK_FILL, GTK_FILL, 3, 3);
 
 	gtk_widget_show_all (ui->vbox);
 
 	/* Set values from the source */
+	property = e_source_get_property (ui->source, "couchdb_database");
+	if (property != NULL)
+		gtk_entry_set_text (GTK_ENTRY (ui->db_name_entry), property);
+	else
+		gtk_entry_set_text (GTK_ENTRY (ui->db_name_entry), "contacts");
+
 	property = e_source_get_property (ui->source, "couchdb_instance");
 	if (g_strcmp0 (property, "system") == 0) {
 		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (ui->system_db_button), TRUE);
@@ -272,10 +301,13 @@ plugin_couchdb_contacts (EPlugin *epl, EConfigHookItemFactoryData *data)
 				    e_source_get_property (ui->source, "couchdb_remote_server"));
 	} else {
 		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (ui->user_db_button), TRUE);
+		if (!property)
+			e_source_set_property (ui->source, "couchdb_instance", "user");
 		gtk_widget_set_sensitive (ui->remote_db_entry, FALSE);
+		gtk_widget_set_sensitive (ui->db_name_entry, FALSE);
 	}
 
-	g_object_set_data_full (G_OBJECT (epl), "gwidget", ui, destroy_ui_data);
+	g_object_set_data_full (G_OBJECT (epl), "cwidget", ui, destroy_ui_data);
 	g_signal_connect (ui->vbox, "destroy", G_CALLBACK (gtk_widget_destroyed), &ui->vbox);
 
 	/* Signals */
@@ -283,12 +315,17 @@ plugin_couchdb_contacts (EPlugin *epl, EConfigHookItemFactoryData *data)
 	g_signal_connect (G_OBJECT (ui->system_db_button), "toggled", G_CALLBACK (on_system_db_toggled), ui);
 	g_signal_connect (G_OBJECT (ui->remote_db_button), "toggled", G_CALLBACK (on_remote_db_toggled), ui);
 	g_signal_connect (G_OBJECT (ui->remote_db_entry), "changed", G_CALLBACK (on_remote_db_changed), ui);
+	g_signal_connect (G_OBJECT (ui->db_name_entry), "changed", G_CALLBACK (on_db_name_changed), ui);
 
 	return NULL;
 }
 
 gint
+#if EDS_MINOR_VERSION < 30
 e_plugin_lib_enable (EPluginLib *ep, gint enable)
+#else
+e_plugin_lib_enable (EPlugin *ep, gint enable)
+#endif
 {
 	if (enable)
 		ensure_couchdb_contacts_source_group ();
